@@ -2,7 +2,7 @@
  *      ipvsadm - IP Virtual Server ADMinistration program
  *                for IPVS NetFilter Module in kernel 2.4
  *
- *      Version: $Id: ipvsadm.c,v 1.23 2003/04/18 09:03:19 wensong Exp $
+ *      Version: $Id: ipvsadm.c,v 1.24 2003/06/08 09:31:36 wensong Exp $
  *
  *      Authors: Wensong Zhang <wensong@linuxvirtualserver.org>
  *               Peter Kese <peter.kese@ijs.si>
@@ -52,6 +52,8 @@
  *        Horms               :   added -v option
  *        Wensong Zhang       :   rewrite most code of parsing options and
  *                                processing options.
+ *        Alexandre Cassen    :   added ipvs_syncd SyncdID support to filter
+ *                                incoming sync messages.
  *
  *
  *      ippfvsadm - Port Fowarding & Virtual Server ADMinistration program
@@ -119,7 +121,7 @@
 #include "libipvs/libipvs.h"
 
 #define IPVSADM_VERSION_NO	"v" VERSION
-#define IPVSADM_VERSION_DATE	"2003/04/16"
+#define IPVSADM_VERSION_DATE	"2003/06/07"
 #define IPVSADM_VERSION		IPVSADM_VERSION_NO " " IPVSADM_VERSION_DATE
 
 #define MAX_TIMEOUT		(86400*31)	/* 31 days */
@@ -179,7 +181,8 @@ static const char* cmdnames[] = {
 #define OPT_THRESHOLDS		0x10000
 #define OPT_PERSISTENTCONN	0x20000
 #define OPT_SORT		0x40000
-#define NUMBER_OF_OPT		19
+#define OPT_SYNCID		0x80000
+#define NUMBER_OF_OPT		20
 
 static const char* optnames[] = {
 	"numeric",
@@ -201,6 +204,7 @@ static const char* optnames[] = {
 	"thresholds",
 	"persistent-conn",
 	"sort",
+	"syncid",
 };
 
 /*
@@ -213,21 +217,21 @@ static const char* optnames[] = {
  */
 static const char commands_v_options[NUMBER_OF_CMD][NUMBER_OF_OPT] =
 {
-	/*   -n   -c   svc  -s   -p   -M   -r   fwd  -w   -x   -y   -mc  tot  dmn  -st  -rt  thr  -pc  srt */
-/*ADD*/     {'x', 'x', '+', ' ', ' ', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*EDIT*/    {'x', 'x', '+', ' ', ' ', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*DEL*/     {'x', 'x', '+', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*FLUSH*/   {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*LIST*/    {' ', '1', '1', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', '1', '1', ' ', ' ', ' ', ' ', ' '},
-/*ADDSRV*/  {'x', 'x', '+', 'x', 'x', 'x', '+', ' ', ' ', ' ', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*DELSRV*/  {'x', 'x', '+', 'x', 'x', 'x', '+', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*EDITSRV*/ {'x', 'x', '+', 'x', 'x', 'x', '+', ' ', ' ', ' ', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*TIMEOUT*/ {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*STARTD*/  {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*STOPD*/   {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*RESTORE*/ {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*SAVE*/    {' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*ZERO*/    {'x', 'x', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+	/*   -n   -c   svc  -s   -p   -M   -r   fwd  -w   -x   -y   -mc  tot  dmn  -st  -rt  thr  -pc  srt  sid */
+/*ADD*/     {'x', 'x', '+', ' ', ' ', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', ' '},
+/*EDIT*/    {'x', 'x', '+', ' ', ' ', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*DEL*/     {'x', 'x', '+', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*FLUSH*/   {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*LIST*/    {' ', '1', '1', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', '1', '1', ' ', ' ', ' ', ' ', ' ', 'x'},
+/*ADDSRV*/  {'x', 'x', '+', 'x', 'x', 'x', '+', ' ', ' ', ' ', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*DELSRV*/  {'x', 'x', '+', 'x', 'x', 'x', '+', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*EDITSRV*/ {'x', 'x', '+', 'x', 'x', 'x', '+', ' ', ' ', ' ', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*TIMEOUT*/ {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*STARTD*/  {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', ' '},
+/*STOPD*/   {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', ' '},
+/*RESTORE*/ {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*SAVE*/    {' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*ZERO*/    {'x', 'x', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
 };
 
 /* printing format flags */
@@ -358,7 +362,7 @@ parse_options(int argc, char **argv, struct ipvs_command_entry *ce,
 		{"restore", 'R', POPT_ARG_NONE, NULL, 'R'},
 		{"save", 'S', POPT_ARG_NONE, NULL, 'S'},
 		{"start-daemon", '\0', POPT_ARG_STRING, &optarg, '1'},
-		{"stop-daemon", '\0', POPT_ARG_NONE, NULL, '2'},
+		{"stop-daemon", '\0', POPT_ARG_STRING, &optarg, '2'},
 		{"tcp-service", 't', POPT_ARG_STRING, &optarg, 't'},
 		{"udp-service", 'u', POPT_ARG_STRING, &optarg, 'u'},
 		{"fwmark-service", 'f', POPT_ARG_STRING, &optarg, 'f'},
@@ -376,6 +380,7 @@ parse_options(int argc, char **argv, struct ipvs_command_entry *ce,
 		{"numeric", 'n', POPT_ARG_NONE, NULL, 'n'},
 		{"connection", 'c', POPT_ARG_NONE, NULL, 'c'},
 		{"mcast-interface", '\0', POPT_ARG_STRING, &optarg, '3'},
+		{"syncid", '\0', POPT_ARG_STRING, &optarg, 'I'},
 		{"timeout", '\0', POPT_ARG_NONE, NULL, '5'},
 		{"daemon", '\0', POPT_ARG_NONE, NULL, '6'},
 		{"stats", '\0', POPT_ARG_NONE, NULL, '7'},
@@ -440,6 +445,11 @@ parse_options(int argc, char **argv, struct ipvs_command_entry *ce,
 		break;
 	case '2':
 		set_command(&ce->cmd, CMD_STOPDAEMON);
+		if (!strcmp(optarg, "master"))
+			ce->daemon.state = IP_VS_STATE_MASTER;
+		else if (!strcmp(optarg, "backup"))
+			ce->daemon.state = IP_VS_STATE_BACKUP;
+		else fail(2, "illegal start_daemon specified");
 		break;
 	case 'h':
 		usage_exit(argv[0], 0);
@@ -548,6 +558,12 @@ parse_options(int argc, char **argv, struct ipvs_command_entry *ce,
 			strncpy(ce->daemon.mcast_ifn,
 				optarg, IP_VS_IFNAME_MAXLEN);
 			break;
+		case 'I':
+			set_option(options, OPT_SYNCID);
+			if ((ce->daemon.syncid =
+			     string_to_number(optarg, 0, 255)) == -1)
+				fail(2, "illegal syncid specified");
+			break;
 		case '5':
 			set_option(options, OPT_TIMEOUT);
 			break;
@@ -637,7 +653,7 @@ parse_options(int argc, char **argv, struct ipvs_command_entry *ce,
 		{"restore", 0, 0, 'R'},
 		{"set", 1, 0, '4'},
 		{"start-daemon", 1, 0, '1'},
-		{"stop-daemon", 0, 0, '2'},
+		{"stop-daemon", 1, 0, '2'},
 		{"tcp-service", 1, 0, 't'},
 		{"udp-service", 1, 0, 'u'},
 		{"fwmark-service", 1, 0, 'f'},
@@ -654,6 +670,7 @@ parse_options(int argc, char **argv, struct ipvs_command_entry *ce,
 		{"numeric", 0, 0, 'n'},
 		{"connection", 0, 0, 'c'},
 		{"mcast-interface", 1, 0, '3'},
+		{"syncid", 1, 0, 'I'},
 		{"timeout", 0, 0, '5'},
 		{"daemon", 0, 0, '6'},
 		{"stats", 0, 0, '7'},
@@ -736,6 +753,11 @@ parse_options(int argc, char **argv, struct ipvs_command_entry *ce,
 		break;
 	case '2':
 		set_command(&ce->cmd, CMD_STOPDAEMON);
+		if (!strcmp(optarg, "master"))
+			ce->daemon.state = IP_VS_STATE_MASTER;
+		else if (!strcmp(optarg, "backup"))
+			ce->daemon.state = IP_VS_STATE_BACKUP;
+		else fail(2, "illegal start_daemon specified");
 		break;
 	case 'h':
 		usage_exit(argv[0], 0);
@@ -821,7 +843,7 @@ parse_options(int argc, char **argv, struct ipvs_command_entry *ce,
 		case 'w':
 			set_option(options, OPT_WEIGHT);
 			if ((ce->dest.weight =
-			     string_to_number(optarg,0,65535)) == -1)
+			     string_to_number(optarg, 0, 65535)) == -1)
 				fail(2, "illegal weight specified");
 			break;
 		case 'x':
@@ -848,6 +870,12 @@ parse_options(int argc, char **argv, struct ipvs_command_entry *ce,
 			strncpy(ce->daemon.mcast_ifn,
 				optarg, IP_VS_IFNAME_MAXLEN);
 			break;
+		case 'I':
+			set_option(options, OPT_SYNCID);
+			if ((ce->daemon.syncid =
+			     string_to_number(optarg, 0, 255)) == -1)
+				fail(2, "illegal syncid specified");
+			break;
 		case '5':
 			set_option(options, OPT_TIMEOUT);
 			break;
@@ -1026,7 +1054,7 @@ static int process_options(int argc, char **argv, int reading_stdin)
 		break;
 
 	case CMD_STOPDAEMON:
-		result = ipvs_stop_daemon();
+		result = ipvs_stop_daemon(&ce.daemon);
 	}
 
 	if (result)
@@ -1247,8 +1275,8 @@ static void usage_exit(const char *program, const int exit_status)
 		"  %s -L|l [options]\n"
 		"  %s -Z [-t|u|f service-address]\n"
 		"  %s --set tcp tcpfin udp\n"
-		"  %s --start-daemon state [--mcast-interface interface]\n"
-		"  %s --stop-daemon\n"
+		"  %s --start-daemon state [--mcast-interface interface] [--syncid sid]\n"
+		"  %s --stop-daemon state\n"
 		"  %s -h\n\n",
 		program, program, program,
 		program, program, program, program, program,
@@ -1291,6 +1319,7 @@ static void usage_exit(const char *program, const int exit_status)
 		"  --u-threshold  -x uthreshold        upper threshold of connections\n"
 		"  --l-threshold  -y lthreshold        lower threshold of connections\n"
 		"  --mcast-interface interface         multicast interface for connection sync\n"
+		"  --syncid sid                        syncid for connection sync (default=255)\n"
 		"  --connection   -c                   output of current IPVS connections\n"
 		"  --timeout                           output of timeout (tcp tcpfin udp)\n"
 		"  --daemon                            output of daemon information\n"
@@ -1739,19 +1768,12 @@ static void list_daemon(void)
 	if (!(u = ipvs_get_daemon()))
 		exit(1);
 
-	switch (u->state) {
-	case IP_VS_STATE_NONE:
-		printf("not running\n");
-		break;
-	case IP_VS_STATE_MASTER:
-		printf("running as master (mcast=%s)\n", u->mcast_ifn);
-		break;
-	case IP_VS_STATE_BACKUP:
-		printf("running as backup (mcast=%s)\n", u->mcast_ifn);
-		break;
-	default:
-		fail(2, "invalid state");
-	}
+	if (u[0].state & IP_VS_STATE_MASTER)
+		printf("master sync daemon (mcast=%s, syncid=%d)\n",
+		       u[0].mcast_ifn, u[0].syncid);
+	if (u[1].state & IP_VS_STATE_BACKUP)
+		printf("backup sync daemon (mcast=%s, syncid=%d)\n",
+		       u[1].mcast_ifn, u[1].syncid);
 	free(u);
 }
 
